/* btmaster.c */ 
#include "kernel.h"
#include "kernel_id.h"
#include "ecrobot_interface.h"

/* OSEK declarations */
DeclareCounter(SysTimerCnt);
DeclareTask(EventDispatcher); 
DeclareTask(EventHandler);
DeclareTask(IdleTask);
DeclareEvent(TouchSensorOnEvent);
DeclareEvent(TouchSensorOffEvent); 

/* below macro enables run-time Bluetooth connection */
#define RUNTIME_CONNECTION
#define TOUCH_COUNT NXT_PORT_S1
#define TURN  NXT_PORT_S2

/* slave device's Bluetooth Device Address (this has to be modified for your NXT) */
const U8 bd_addr[7] = {0x00, 0x16, 0x53, 0x10, 0x3C, 0x08, 0x00};


/*Global Variables*/
int touch=1,touch_1=1;
U8 touch_counter=0,turn=0;

/* LEJOS OSEK hooks */
void ecrobot_device_initialize()
{
#ifndef RUNTIME_CONNECTION
	ecrobot_init_bt_master(bd_addr, "LEJOS-OSEK");
#endif
}
void ecrobot_device_terminate()
{
	ecrobot_term_bt_connection();
}

/* LEJOS OSEK hook to be invoked from an ISR in category 2 */
void user_1ms_isr_type2(void)
{
  StatusType ercd;

  ercd = SignalCounter(SysTimerCnt); /* Increment OSEK Alarm Counter */
  if(ercd != E_OK)
  {
    ShutdownOS(ercd);
  }
}

/* EventDispatcher executed every 5ms */
TASK(EventDispatcher)
{
  static U8 bt_receive_buf[32]; 
  static U8 TouchSensorStatus_old = 0;
  U8 TouchSensorStatus; 


/*Touch counter for secondary colour port on slave nxt*/
  if(ecrobot_get_touch_sensor(TOUCH_COUNT))
   {
	if(touch)
	touch_counter++;
        touch=0;
   }
 else
      touch=1;


/*Counter for turn 1-left 2-right 0-default*/
  if(ecrobot_get_touch_sensor(TURN))
   {
	if(touch_1)
	turn++;
        touch_1=0;
   }
 else
      touch_1=1;


//display values for debug
      display_goto_xy(10,0);
      display_int(touch_counter,0); // display the current sensor value
      display_goto_xy(10,1);
      display_int(turn,0); // display the current sensor value
      display_update();
//Logic event enable for one shot
  TouchSensorStatus = ecrobot_get_touch_sensor(NXT_PORT_S4);
  if (TouchSensorStatus == 1 && TouchSensorStatus_old == 0)
  {
    /* Send a Touch Sensor ON Event to the Handler */ 
    SetEvent(EventHandler, TouchSensorOnEvent);
  }
  else if (TouchSensorStatus == 0 && TouchSensorStatus_old == 1)
  {
    /* Send a Touch Sensor OFF Event to the Handler */ 
    //SetEvent(EventHandler, TouchSensorOffEvent);
  }
  TouchSensorStatus_old = TouchSensorStatus;

  TerminateTask();
}

/* EventHandler executed by OSEK Events */
TASK(EventHandler)
{
  static U8 bt_send_buf[32]; 

  while(1)
  {
    WaitEvent(TouchSensorOnEvent); /* Task is in waiting status until the Event comes */ 
    ClearEvent(TouchSensorOnEvent);
  	/* send packet data to the slave device */
  	bt_send_buf[0] = touch_counter;//stop point
	bt_send_buf[1] = turn; //left or right
	ecrobot_send_bt_packet(bt_send_buf, 32);
//debug value displayed
      display_goto_xy(10,2);
      display_int(bt_send_buf[0],0); // display the current sensor value
      display_goto_xy(10,3);
      display_int(bt_send_buf[1],0); // display the current sensor value
      display_update();

    WaitEvent(TouchSensorOffEvent); /* Task is in waiting status until the Event comes */
    ClearEvent(TouchSensorOffEvent);
  	/* send packet data to the slave device */
  	bt_send_buf[0] = 0;
	ecrobot_send_bt_packet(bt_send_buf, 32);
  }

  TerminateTask();
}

/* IdleTask */
TASK(IdleTask)
{
  static SINT bt_status = BT_NO_INIT;

  while(1)
  {  
#ifdef RUNTIME_CONNECTION
    ecrobot_init_bt_master(bd_addr, "LEJOS-OSEK");
#endif

    if (ecrobot_get_bt_status() == BT_STREAM && bt_status != BT_STREAM)
    {
      display_clear(0);
      display_goto_xy(0, 0);
      display_string("[BT]");
      display_update();
    }
    bt_status = ecrobot_get_bt_status();
  }	
}
